#include <Class_FileDialog.hpp>
#include "Class_SafeDocumentHandling.hpp"

//------------------------------------------------------------------------

Class_SafeDocumentHandling::Class_SafeDocumentHandling( HWND _hwnd, PSZ szApplicationName )
{
     // Set the handle to the window
     hwnd = _hwnd;

     // Set the name of the application
     if( szApplicationName )
     {
          // szApplicationName is not NULL
          pszApplicationName = new char [ strlen( szApplicationName ) + 1 ];
          strcpy( pszApplicationName, szApplicationName );
     }
     else
     {
          // The name of the application was not supplied
          pszApplicationName = NULL;
     }

     // Initialize the filename of the document
     strcpy( szFilename, "" );

     // Initialize the bModifiedDocument member
     bModifiedDocument = FALSE;

     // Set the titlebar text
     UpdateTitlebar();
}

//------------------------------------------------------------------------

Class_SafeDocumentHandling::~Class_SafeDocumentHandling()
{
     // Last chance to save the work
     // This last check shouldn't be needed in a well written program
     CanClose( FALSE );

     // Release allocated memory
     if( pszApplicationName )
     {
          delete [] pszApplicationName;
     }
}

//------------------------------------------------------------------------

BOOL Class_SafeDocumentHandling::CanClose( BOOL bCanCancel )
{
     // Determines whether the current document can be safely closed

     if( !bModifiedDocument )
     {
          // We can proceed since the document is unchanged
          return TRUE;
     }

     ULONG ulResponse;
     // Obtain a response from the user
     // If the user can cancel (default), then a CANCEL button is displayed
     // If the user cannot cancel, then the CANCEL button is not displayed
     ulResponse = WinMessageBox(HWND_DESKTOP,
              HWND_DESKTOP,
              "Do you want to save your work?",
              "Document Modified",
              0,
              ( bCanCancel ? MB_YESNOCANCEL : MB_YESNO ) | MB_ICONQUESTION | MB_MOVEABLE);

     switch( ulResponse )
     {
          case MBID_YES:
               // Save the work
               return FileSave();
          case MBID_NO:
               // Discard the work
               bModifiedDocument = FALSE;
               return TRUE;
          case MBID_CANCEL:
               // Cancel the action
               return FALSE;
     }
     return FALSE;
}

//------------------------------------------------------------------------

BOOL Class_SafeDocumentHandling::FileSave()
{
     // Saves the data from the current document

     if ( strlen( szFilename ) )
     {
          // We have a valid filename
          if( Save() )
          {
               // Successfully saved the document
               bModifiedDocument = FALSE;
               UpdateTitlebar();
               return TRUE;
          }
          else
          {
               return FALSE;
          }
     }
     else
     {
          // We do not have a valid filename
          // So we call FileSaveAs() to obtain one
          return FileSaveAs();
     }
}

//------------------------------------------------------------------------

BOOL Class_SafeDocumentHandling::FileSaveAs( BOOL bVerify )
{
     // Requests a filename from the user and saves the document
     // When bVerify is TRUE, the selected file will be verified to
     // prevent overwritting it

     // Display a file Save as dialog to retrieve a filename
     Class_FileSaveAs Dlg( szFilename );

     // Execute the dialog
     if( Dlg.Do( szFilename, bVerify ) )
     {
          // A valid filename was retrieved
          return FileSave();
     }
     else
     {
          // The user clicked CANCEL or closed the dialog,
          // so we did not retrieve a filename
          return FALSE;
     }
}

//------------------------------------------------------------------------

BOOL Class_SafeDocumentHandling::FileNew()
{
     // Creates a new document
     if( CanClose() )
     {
          // Reset the name for the document
          strcpy( szFilename, "");
          UpdateTitlebar();
          return TRUE;
     }
     else
     {
          // The user wants to keep the current document
          return FALSE;
     }
}

//------------------------------------------------------------------------

BOOL Class_SafeDocumentHandling::FileOpen( BOOL bVerify )
{
     // Opens a single document
     // If bVerify is TRUE, then the selected file will be
     // verified for existence

     // Opening multiple documents is not supported here since
     // it would depend on the implementation:  Multiple Document Interface (MDI),
     // rings (as in EPM), or items in a menu (DOS's Edit).  Also, multiple selections
     // could be used as a sequence of input files (the slideshow option of PMJPEG).
     // But I might work on this, and figure out an EASY way of implementing this

     if( CanClose() )
     {
          // We can load a new document
          Class_FileOpen Dlg( szFilename );
          if( Dlg.Do( szFilename, bVerify ) )
          {
               // Save the filename because FileNew() will destroy it
               // This is to permit a user to Cancel a file open and return
               // to the previous work.
               char szTempFilename[CCHMAXPATH];
               strcpy( szTempFilename, szFilename );

               // We retrieved a filename, so let's discard the old work
               if( FileNew() )
               {
                    // The state of the application was reset
                    // We can proceed by loading the new document

                    // Restore the filename
                    strcpy( szFilename, szTempFilename );
                    if( Load() )
                    {
                         // Successfully loaded the file
                         UpdateTitlebar();
                         return TRUE;
                    }
               }
          }
     }

     // The user wants to keep the current document
     // Or an error occurred during the load of the file
     return FALSE;
}

//------------------------------------------------------------------------

void Class_SafeDocumentHandling::SetModified()
{
     // This function indicates to the safe document handling scheme
     // that the document has been changed.
     // It could be improved upon by allowing users to also set the
     // bModifiedDocument variable to FALSE, in order to support an undo
     // feature.  But I don't particularly like this idea:  the document has
     // been worked on, even if UNDO's return it to its original state.

     if( !bModifiedDocument )
     {
          // The document was not previously modified
          // so we perform the work (slight optimization)
          bModifiedDocument = TRUE ;
          UpdateTitlebar();
     }
}

//------------------------------------------------------------------------

void Class_SafeDocumentHandling::UpdateTitlebar()
{
     // Sets the title of the specified window
     // The following format is used:
     //   application - document
     // If the document has been modified, a * is appended
     // The titlebar text will be updated automatically by these actions
     //        FileNew(), FileSave(), FileSaveAs(), FileOpen(), SetModified()

     if( !hwnd )
     {
          // There's no point in going any further since
          // the handle to the window is NULL and the
          // WinSetWindowText() function will not work
          // without it.  Maybe this could be improved by
          // querying the system for the handle of the window
          return;
     }

     int length = 0;// Length of the titlebar text

     if( strlen( pszApplicationName ) )
          // If an application name was supplied, we also add
          // room for ' - ', an appended '*' and a trailing '\0'
          length = strlen( pszApplicationName ) + 5;

     if( strlen( szFilename ) )
          length += strlen( szFilename ) + 1;// Don't forget the trailing '\0'
     else
          // Use 'Untitled' as the filename
          length += strlen( "Untitled" ) + 1;// Don't forget the trailing '\0'

     char *pszText = new char [ length ];// String to hold the text
     if( strlen( pszApplicationName ) )
          sprintf( pszText,
                   "%s - %s%c",
                   pszApplicationName,
                   strlen( szFilename ) ? szFilename : "Untitled",
                   bModifiedDocument ? '*' : ' ' );
     else
          sprintf( pszText,
                   "%s%c",
                   strlen( szFilename ) ? szFilename : "Untitled",
                   bModifiedDocument ? '*' : ' ' );

     WinSetWindowText( hwnd, pszText );

     delete [] pszText;// Release allocated memory
}

//------------------------------------------------------------------------

PSZ Class_SafeDocumentHandling::Get_Filename()
{
     // Returns the filename
     return szFilename;
}

//------------------------------------------------------------------------


